'use strict';

var Router = require('koa-router');
var VulnerabilityModel = require('vulnerabilityModel');
var mongoose = require('mongoose');
var ObjectId = mongoose.Types.ObjectId;

var router = new Router({
    prefix: '/vulnerability'
});


class VulnerabilityAPI {

    static * list() {

        try {
            this.body = yield VulnerabilityModel.find({}).exec();

        } catch(e) {
            console.error(e);
            this.throw('Generic error', 500);
        }
    }

    static * get() {
        this.assert(this.params.id, 400, 'id param required');
        try {
            var vulnerability = yield VulnerabilityModel.findOne({_id:ObjectId(this.params.id) }).exec();
            if(!vulnerability){
                this.throw(404, 'Vulnerability not found');
            }else{
                this.body = vulnerability;
            }
        } catch(e) {
            console.error(e);
            this.throw('Generic error', 500);
        }
    }

    static * create() {
        this.assert(this.request.body, 400, 'Vulnerability not found');
        try {

            console.log(JSON.stringify(this.request.body));
            var vuln = new VulnerabilityModel(this.request.body);
            var result = yield vuln.save();
            this.body = result;

        } catch(e) {
            console.error(e);
            this.throw('Generic error', 500);
        }
    }

    static * update() {
        this.assert(this.params.id, 400, 'Id not found');
        this.assert(this.request.body, 400, 'Body required');
        try {
            var vulnerability = yield VulnerabilityModel.findOne({_id:ObjectId(this.params.id) }).exec();
            if(!vulnerability){
                this.throw(404, 'Vulnerability not found');
            }
            vulnerability.name = this.request.body.name;
            vulnerability.description = this.request.body.description;
            vulnerability.level = this.request.body.level;
            vulnerability.type = this.request.body.type;
            yield vulnerability.save();
            this.body = vulnerability;

        } catch(e) {
            console.error(e);
            this.throw('Generic error', 500);
        }
    }

    static * delete() {
        this.assert(this.params.id, 400, 'Id not found');
        try {
            yield VulnerabilityModel.remove({_id:ObjectId(this.params.id) }).exec();

            this.status= 204;
            this.body = '';

        } catch(e) {
            console.error(e);
            this.throw('Generic error', 500);
        }
    }


}


router.get('/', VulnerabilityAPI.list);
router.get('/:id', VulnerabilityAPI.get);
router.patch('/:id', VulnerabilityAPI.update);
router.delete('/:id', VulnerabilityAPI.delete);
router.post('/', VulnerabilityAPI.create);

module.exports = router;
